USE master;
GO
IF DB_ID('UDF_Tests') IS NOT NULL
BEGIN
	ALTER DATABASE UDF_Tests SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE UDF_Tests;
END

CREATE DATABASE UDF_Tests
GO
USE UDF_Tests;
GO

-- Example from Hugo Kornelis UDF blog series Part 1
-- http://sqlblog.com/blogs/hugo_kornelis/archive/2012/05/20/t-sql-user-defined-functions-the-good-the-bad-and-the-ugly-part-1.aspx
-- http://sqlblog.com/blogs/hugo_kornelis/archive/2012/05/26/t-sql-user-defined-functions-the-good-the-bad-and-the-ugly-part-2.aspx
-- http://sqlblog.com/blogs/hugo_kornelis/archive/2012/06/29/t-sql-user-defined-functions-the-good-the-bad-and-the-ugly-part-3.aspx
-- http://sqlblog.com/blogs/hugo_kornelis/archive/2012/08/25/t-sql-user-defined-functions-the-good-the-bad-and-the-ugly-part-4.aspx

CREATE TABLE dbo.LargeTable 
  (KeyVal int NOT NULL PRIMARY KEY, 
   DataVal int NOT NULL CHECK (DataVal BETWEEN 1 AND 10) 
  );

WITH Digits 
AS (SELECT d FROM (VALUES (0),(1),(2),(3),(4),(5),(6),(7),(8),(9)) AS d(d)) 
INSERT INTO dbo.LargeTable (KeyVal, DataVal) 
SELECT 10000 * tt.d + 1000 * st.d 
     + 100 * h.d + 10 * t.d + s.d + 1, 
       10 * RAND(CHECKSUM(NEWID())) + 1 
FROM   Digits AS s,  Digits AS t,  Digits AS h, 
       Digits AS st, Digits AS tt;
GO

CREATE FUNCTION dbo.Triple(@Input int) 
       RETURNS int 
AS 
BEGIN; 
  DECLARE @Result int; 
  SET @Result = @Input * 3; 
  RETURN @Result; 
END; 
GO


-- Test 1 duration
SET NOCOUNT ON

DECLARE @StartTime DATETIME
SET @StartTime = GETDATE()

SELECT dbo.Triple(DataVal) AS Triple 
FROM   dbo.LargeTable;

PRINT 'Duration with scalar functions: ' + CAST(DATEDIFF(ms, @StartTime, GETDATE()) AS varchar(10)) + ' ms'

SET @StartTime = GETDATE()

SELECT 3 * DataVal AS MaxTriple 
FROM   dbo.LargeTable;

PRINT 'Duration with inline operation: ' + CAST(DATEDIFF(ms, @StartTime, GETDATE()) AS varchar(10)) + ' ms'

GO


-- Test 2 STATISTICS TIME
SET STATISTICS TIME ON

DECLARE @StartTime DATETIME
SET @StartTime = GETDATE()

SELECT dbo.Triple(DataVal) AS Triple 
FROM   dbo.LargeTable;

PRINT 'Duration with scalar functions: ' + CAST(DATEDIFF(ms, @StartTime, GETDATE()) AS varchar(10)) + ' ms'

SET @StartTime = GETDATE()

SELECT 3 * DataVal AS MaxTriple 
FROM   dbo.LargeTable;

PRINT 'Duration with inline operation: ' + CAST(DATEDIFF(ms, @StartTime, GETDATE()) AS varchar(10)) + ' ms'

SET STATISTICS TIME OFF
GO


-- Make the table larger to 10,000,000 rows
TRUNCATE TABLE dbo.LargeTable;

WITH Digits 
AS (SELECT d FROM (VALUES (0),(1),(2),(3),(4),(5),(6),(7),(8),(9)) AS d(d)) 
INSERT INTO dbo.LargeTable (KeyVal, DataVal) 
SELECT 1000000 * sm.d 
     + 100000 * ht.d + 10000 * tt.d + 1000 * st.d 
     + 100 * h.d + 10 * t.d + s.d + 1, 
       10 * RAND(CHECKSUM(NEWID())) + 1 
FROM   Digits AS s,  Digits AS t,  Digits AS h, 
       Digits AS st, Digits AS tt, Digits AS ht, 
       Digits AS sm;
       
GO




-- Test 3 STATISTICS TIME again
SET STATISTICS TIME ON

DECLARE @StartTime DATETIME
SET @StartTime = GETDATE()

SELECT dbo.Triple(DataVal) AS Triple 
FROM   dbo.LargeTable;

PRINT 'Duration with scalar functions: ' + CAST(DATEDIFF(ms, @StartTime, GETDATE()) AS varchar(10)) + ' ms'

SET @StartTime = GETDATE()

SELECT 3 * DataVal AS MaxTriple 
FROM   dbo.LargeTable;

PRINT 'Duration with inline operation: ' + CAST(DATEDIFF(ms, @StartTime, GETDATE()) AS varchar(10)) + ' ms'

SET STATISTICS TIME OFF
GO



-- Test 4 STATISTICS TIME again - No client processing - Collect Actual Execution Plans
SET STATISTICS TIME ON

DECLARE @MaxVal int;
   
DECLARE @StartTime DATETIME
SET @StartTime = GETDATE()

SELECT @MaxVal = MAX(dbo.Triple(DataVal))
FROM   dbo.LargeTable;

PRINT 'Duration with scalar functions: ' + CAST(DATEDIFF(ms, @StartTime, GETDATE()) AS varchar(10)) + ' ms'

SET @StartTime = GETDATE()

SELECT @MaxVal = MAX(3 * DataVal)
FROM   dbo.LargeTable;

PRINT 'Duration with inline operation: ' + CAST(DATEDIFF(ms, @StartTime, GETDATE()) AS varchar(10)) + ' ms'

SET STATISTICS TIME OFF
GO